{ lib, config, pkgs, ... }: {
  imports = [

    <system/desktop>
    ./hardware-configuration.nix

    ./kibana.nix
    ./packages.nix
    ./slack.nix
    ./syncthing.nix
    ./tinc.nix

  ];

  networking.hostName = "workout";

  services.logind.lidSwitch = lib.mkForce "ignore";

  security.wrappers = {
    pmount.source = "${pkgs.pmount}/bin/pmount";
    pumount.source = "${pkgs.pmount}/bin/pumount";
  };

  # nix-shell -p speedtest_cli --run speedtest
  configuration.fireqos = {
    enable = true;
    interface = "wlp1s0";
    input = 4200;
    output = 1200;
    balance = false;
  };

  programs.custom.steam.enable = true;
  programs.custom.video.enable = true;

  # font
  # ----
  programs.custom.urxvt.fontSize = 17;
  programs.custom.xterm.fontSize = 17;
  system.custom.fonts.dpi = 140;

  services.printing.enable = true;

  virtualisation = {
    docker.enable = true;
    virtualbox = {
      host.enable = true;
      guest.x11 = true;
      guest.enable = true;
    };
  };

  configuration.desktop = {
    width = 2560;
    height = 1440;
  };

}

