{ config, lib, pkgs, ... }:

let mainUserHome = "/home/palo";
in {
  # fix fileSystems.<name>.encrypted - false overwrite
  # --------------------------------------------------
  boot.initrd.luks.cryptoModules = [
    "aes"
    "aes_generic"
    "blowfish"
    "twofish"
    "serpent"
    "cbc"
    "xts"
    "lrw"
    "sha1"
    "sha256"
    "sha512"
    "aes_x86_64"
  ];

  # todo : why should I use this here
  boot.initrd.availableKernelModules =
    [ "xhci_pci" "ahci" "usb_storage" "sd_mod" ];
  boot.kernelModules = [ "kvm-intel" ];
  nix.maxJobs = lib.mkDefault 8;

  # lvm volume group
  # ----------------
  boot.initrd.luks.devices = [{
    name = "vg";
    device = "/dev/sda2";
    preLVM = true;
  }];

  # NTFS support
  # ------------
  environment.systemPackages = [ pkgs.ntfs3g ];

  # root
  # ----
  fileSystems."/" = {
    options = [ "noatime" "nodiratime" "discard" ];
    device = "/dev/vg/root";
    fsType = "ext4";
  };

  # boot
  # ----
  fileSystems."/boot" = {
    device = "/dev/sda1";
    fsType = "ext4";
  };
  boot.loader.grub = {
    device = "/dev/sda";
    enable = true;
    version = 2;
  };

  # home
  # ----
  fileSystems."/home" = {
    options = [ "noatime" "nodiratime" "discard" ];
    device = "/dev/vg/home";
    fsType = "ext4";
  };

}

